/*
 *
 * LED Panel Dashboard widget for Mac OS X
 * v0.8.5 (c) 2009.01.15 Zachary.Johnson 
 * http://zachstronaut.com
 *
 */


/*
0000000000000000
0111000001111100
0011000001001100
0011000001001100
0011000001001100
0011001101001100
0111101101111100
0000000000000000
0000000000000000
0100001110111000
0100001000100100
0100001000100100
0100001100100100
0100001000100100
0111101110111000
0000000000000000
*/

window.onload = setup;

var infoButton;
var glassLoadURLButton;
var glassCancelButton;

var panelObj;

var comboBox;



function setup()
{
    // Instantiate
    panelObj = new Panel( 'panel', 'front', 'back', 'url', 'error', 'version' );

    // These features work just fine in Safari, so I'm not testing for window.widget
    infoButton = new AppleInfoButton( $('infoButton'), $('front'), 'white', 'white', panelObj.showPrefs );
	glassLoadURLButton = new AppleGlassButton( $('loadURLButton'), 'Load URL', loadButton );
	glassCancelButton = new AppleGlassButton( $('cancelButton'), 'Cancel', panelObj.hidePrefs );

    // Setup combo box on the flip side of our widget
    comboBox = new ComboBox( 'url', 'choices' );
    $('arrow').onclick = function () { $('url').focus(); };
    
    // Set some defaults
    var url = 'led:intro';
    var h = 16;
    var v = 16;
    
    // Look for saved values to override defaults
    // And setup some other widgety type stuff
    if ( window.widget )
    {
        url = widget.preferenceForKey( makeUniqueKey( 'url' ) ) || url;
        
        // Callback for when this instance of the widget is removed from the Dashboard
        widget.onremove = clearPrefs;
        
        // Callback for when the Dashboard is hidden
        widget.onhide = onHide;
        
        // Callback for when the Dashboard is shown
        widget.onshow = onShow;
    }

    var back = $('back');
    back.ondrop = onDrop;
    back.ondragenter = onDragEnter;
    back.ondragover = onDragOver;
    back.ondragleave = onDragLeave;
    
    var front = $('front');
    front.ondrop = onDrop;
    front.ondragenter = onDragEnter;
    front.ondragover = onDragOver;
    front.ondragleave = onDragLeave;

    // Init our panel
    panelObj.init( h, v );
    $('url').value = url;
    
    // When the widget is first created we want to give a little pause before
    // we hit the hardcore processing.  If we start right away without waiting
    // a tick, the user won't even see it.
    setTimeout( panelObj.processUrlInput, 0 );
}


function onDrop( event )
{
    var uri;
    
    try
    {
        uri = event.dataTransfer.getData( 'text/uri-list' );
    }
    catch ( e )
    {}
	
	if ( uri )
	{
	    // Adjust for dragged scripts vs. dragged static pages
	    if ( uri.search( /\.led$/ ) >= 0 )
	    {
	       uri = uri.replace( /^file:\/\/localhost/, 'script:' );
	    }

	    $('url').value = uri;
	    
	    loadButton();
    }
    
    event.stopPropagation();
	event.preventDefault();
}
function onDragEnter( event )
{
	event.stopPropagation();
	event.preventDefault();
}
function onDragOver( event )
{
	event.stopPropagation();
	event.preventDefault();
}
function onDragLeave( event )
{
	event.stopPropagation();
	event.preventDefault();
}


function onHide()
{
    if ( !panelObj.persistent )
    {
        panelObj.pause();
    }
}


function onShow()
{
    panelObj.resume();
}


function loadButton()
{
    if ( window.widget )
    {
        savePrefs();
    }
    
    panelObj.processUrlInput();
}


function savePrefs()
{
    widget.setPreferenceForKey( $('url').value, makeUniqueKey( 'url' ) );
    
    //doUrlHistory();
}


function clearPrefs()
{
    widget.setPreferenceForKey( null, makeUniqueKey( 'url' ) );
}


/*
function doUrlHistory()
{
    var url = $('url').value;
    
    if ( url.search( /^file:\/\// ) >= 0 )
    {
        comboBox.removeChoice( 'file://' );
        comboBox.prependChoice( url );
        
        //widget.setPreferenceForKey( url, 'history1' );
    }
}
*/


//
// Uses each widget's unique identifier to create a key specific to this
// widget instance when calling widget.setPreferenceForKey()
//
function makeUniqueKey( key )
{
	return widget.identifier + '-' + key;
}