/*
 *
 * ComboBox control for static input value suggestions
 * v0.9.2 (c) 2006 Zachary.Johnson
 * http://zachstronaut.com/
 *
 */

//
// ComboBox class definition
//
function ComboBox( inputId, divId, selectedClass )
{
    //
    // Class Vars
    //
    
    // Our HTML elements
    this.inputElement = $( inputId );
    this.divElement = $( divId );
    
    // Class to apply to choice selected by keyboard
    this.selectedClass = ( selectedClass ) ? selectedClass : 'selected';

    // Is our mouse over our choices menu?
    // We get a blur event before our click event, so we want to ignore
    // an onblur if the mouse is over our choices menu
    this.mouseOverChoices = 0;
    
    // Which choice is selected (by keyboard)
    this.selected = -1;
    
    // An array of <li> elements representing our choices
    this.choices = this.divElement.childNodes[0].childNodes;
    

    // Apparently eliminates scope problems in timers/event handlers
    var self = this;


    this.inputElement.onfocus = function () { self.showChoices(); };
    
    
    this.inputElement.onblur = function () { if ( !self.mouseOverChoices ) { self.hideChoices(); } };
    
    
    this.inputElement.onkeypress =  function ( event )
                                    {
                                        switch ( event.keyCode )
                                        {
                                            //case 38:
                                            case 63232: //KEY_UP
                                                self.selectPrev();
                                                return false;

                                            //case 40:
                                            case 63233: //KEY_DOWN
                                                if ( self.selected == -1 )
                                                {
                                                    self.showChoices();
                                                }

                                                self.selectNext();
                                                return false;

                                            case 13: //KEY_RETURN
                                                if ( self.selected > -1 )
                                                {
                                                    self.selectChoice( self.choices[self.selected] );
                                                }
                                                else
                                                {
                                                    self.hideChoices();
                                                }
                                                return false;
                                        }
                                    };
    
    
    this.divElement.onmouseover = function () { self.mouseOverChoices = 1; };
    
    
    this.divElement.onmouseout = function () { self.mouseOverChoices = 0; };


    for ( var i = 0; i < this.choices.length; i++ )
    {
        this.choices[i].onclick = function () { self.selectChoice( this ); };
    }
    

    //
    // Methods
    //
    
    this.selectPrev = function ()
    {
        if ( self.selected > 0 )
        {
            self.selected--;
            self.showSelected();
        }
    }


    this.selectNext = function ()
    {
        if ( self.selected < self.choices.length - 1 )
        {
            self.selected++;
            self.showSelected();
        }
    }


    this.showSelected = function ()
    {
        for ( var i = 0; i < self.choices.length; i++ )
        {
            if ( i == self.selected )
            {
                self.choices[i].className = self.selectedClass;
            }
            else
            {
                self.choices[i].className = '';
            }
        }
    }


    this.showChoices = function ()
    {
        self.divElement.style.display = 'block';

        // Clears out highlight from last time
        self.selected = -1;
        self.showSelected();
    }


    this.hideChoices = function ()
    {
        self.selected = -1;
        self.mouseOverChoices = 0;
        self.divElement.style.display = 'none';
    }


    this.selectChoice = function ( element )
    {
        self.hideChoices();
        self.inputElement.value = element.innerHTML;
    }
    
    
    this.removeChoice = function ( choiceText )
    {
        for ( var i = 0; i < self.choices.length; i++ )
        {
            if ( self.choices[i].innerHTML == choiceText )
            {
                self.divElement.childNodes[0].removeChild( self.choices[i] );
                return true;
            }
        }
        
        return false;
    }
    
    
    this.addChoiceAfter = function ( n, choiceText )
    {
        var li = document.createElement( 'li' );
        li.innerHTML = choiceText;
        
        // Add after last element
        if ( n >= self.choices.length - 1 )
        {
            self.divElement.childNodes[0].appendChild( li );
        }
        // All other adds
        // Negative n (-1) means add at start
        else
        {
            if ( n < -1 )
            {
                n = -1;
            }
            
            self.divElement.childNodes[0].insertBefore( li, self.choices[ n + 1 ] );
        }
    }
    
    
    this.prependChoice = function ( choiceText )
    {
        self.addChoiceAfter( -1, choiceText );
    }
    
    
    this.appendChoice = function( choiceText )
    {
        self.addChoiceAfter( self.choices.length, choiceText );
    }
}
