/*
 *
 * (c) 2006-2009 Zachary.Johnson
 * http://zachstronaut.com
 *
 */

//
// Panel class definition
//
function Panel( panelId, frontId, backId, urlId, errorId, versionId )
{
    //
    // Class Vars
    //
    
    // Current version
    this.version = '0.8.5';
    
    this.clri = 0;
    this.LEDs = null;
    this.states = null;
    this.hcount = 0;
    this.vcount = 0;
    
    // Our HTML elements
    this.panelElement = $( panelId );
    this.frontElement = $( frontId );
    this.backElement = $( backId );
    this.urlElement = $( urlId );
    this.errorElement = $( errorId );
    this.versionElement = $( versionId );

    // Display current version
    this.versionElement.innerHTML = 'LED Panel ' + this.version;

    // Our AJAX object
    this.XML_Request = null;
    
    // AJAX object for new version checking
    this.XML_VerReq = null;
    
    // Our widget.system object
    this.systemObj = null;
    
    // When did we last check our version?
    this.verLastChecked = null;
    
    // Specifically for built in clock
    this.clockOn = 0;
    
    // Specifically for built in scroll
    this.scrollOn = 0;
    
    // Timer object used for keying up next url action
    this.loopTimer = null;
    
    // URL to load in the event of an LED click
    this.onclickURL = null;
    
    // The zero-indexed number of the LED that was clicked
    this.clickedLEDindex = -1;
    
    // We store what would be our next action here.  If the Dashboard is
    // hidden we will use this action to resume processing once the
    // Dashboard is shown again.
    this.nextAction = function () {};
    
    // If the Panel is in persistent mode than it will not be paused when
    // the Dashboard is hidden.
    this.persistent = false;
    
    // State indicated that the panel has been paused
    this.paused = false;
    
    // Apparently eliminates scope problems in timers/event handlers
    var self = this;
    
    
    //
    // Methods
    //
    
    this.init = function ( hcount, vcount )
    {
        // If we have any LEDs, get rid of them
        if ( self.LEDs )
        {
            // Memory leak?  Not sure how to loop through and do a removeChild?
            self.panelElement.innerHTML = '';
        }
        
        self.LEDs = new Array();
        
        self.hcount = hcount;
        self.vcount = vcount;
        self.clri = -1 * self.hcount;
        
        // Create our LED elements and local array
        for ( var i = 0; i < hcount * vcount; i++ )
        {
            var div = document.createElement( 'div' );
            div.setAttribute( 'class', 'LED' );
            div.onclick = ( function ( i ) { return function () { self.clickHandler( i ); }; } )( i );
            self.panelElement.appendChild( div );
            self.LEDs.push( div );
        }
        
        // Init LED states
        self.states = new Array( self.LEDs.length );

        // Set the css size of our panel
        self.panelElement.style.width = ( self.hcount * 20 ).toString() + 'px';
        self.panelElement.style.height = ( self.vcount * 20 ).toString() + 'px';
        
        // Set the size of our widget
        if ( window.widget )
        {            
            // window.resizeTo causes the ripple effect to stop prematurely,
            // and there's no harm in being sized too large... so except for
            // really big panels, we just size our window the size size of
            // Default.png so the ripple be seen in its full glory.
            var h;
            var v;
            
            if ( self.hcount <= 16 )
            {
                h = 365;
            }
            else
            {
                h = ( self.hcount * 20 ) + 45;
            }
            
            if ( self.vcount <= 16 )
            {
                v = 365;
            }
            else
            {
                v = ( self.vcount * 20 ) + 45;
            }
            
            window.resizeTo( h, v );
        }
    }
    
    
    this.checkVersion = function ()
    {
        // We're only checking once per hour.
        if ( !self.verLastChecked || ( new Date() ) - self.verLastChecked > 60 * 60 * 1000 )
        {
            self.verLastChecked = new Date();
            
            if ( self.XML_VerReq )
            {
                self.XML_VerReq.abort();
            }
            
            self.XML_VerReq = new XMLHttpRequest();
            self.XML_VerReq.onreadystatechange = function () { self.versionResponseHandler() };
            self.XML_VerReq.open( 'GET', 'http://led.zachstronaut.com/version.txt?t=' + self.verLastChecked.getMilliseconds().toString() + Math.round( Math.random() * 10000 ).toString(), true );
            self.XML_VerReq.send( null );
        }
    }
    
    
    this.versionResponseHandler = function ()
    {
         if ( self.XML_VerReq.readyState == 4 && self.XML_VerReq.status == 200 && self.XML_VerReq.responseText )
         {
             if ( self.version != self.XML_VerReq.responseText.split( "\n" )[0] )
             {
                 self.versionElement.innerHTML = 'New Version Available!';
             }
         }
    }
    
    
    // Handle a new URL put into UI input field
    this.processUrlInput = function ()
    {
        self.hideError();
        
        self.processUrl( self.urlElement.value, 1 );
    }
    
    
    this.processUrl = function ( url, isNew )
    {
        var action = null;
        var sweep = 1;
        
        // pass in everything up to the first space
        // this will allow us to case out all the built ins
        // and then urls and scripts fit nicely into the default
        switch ( url.toLowerCase().split( /\s/ )[0] ) {
            case 'led:time':
            case 'led:clock':
                action = function () { self.clockInit(); };
                break;

            case 'led:blank':
            case 'led:clear':
                if ( isNew )
                {
                    // already clears
                    action = function () {};
                }
                else
                {
                    action = function () { self.clear(); };
                }
                break;
            
            case 'led:stop':
            case 'led:pause':
                self.pause();
                self.hidePrefs();
                return;
            
            case 'led:start':
            case 'led:resume':
                self.resume();
                self.hidePrefs();
                return;

            case 'led:intro':
                action = function () { self.intro(); };
                break;

            case 'led:logo':
            case 'led:invader':
                action = function () { self.invader(); };
                break;

            case 'led:scroll':
                var bits = url.split( /\s+/ );

                var delay = parseInt( bits[1] );
                if ( !delay || Math.abs( delay ) < 10 )
                {
                    delay = 50;
                }
                
                var duration = parseInt( bits[2] );
                if ( !duration )
                {
                    duration = 0;
                }

                action = function () { self.scrollInit( delay, duration ); };
                sweep = 0;
                break;
            
            case 'led:paint':
                self.processUrl( url.replace( /^led:paint/, 'script:Extras/paint.led' ), isNew );
                return;
            
            case 'led:marquee':
                self.processUrl( url.replace( /^led:marquee/, 'script:Extras/ticker.led' ), isNew );
                return;
            
            default:    
                //
                // Handle local scripts and local/remote URLs
                //
                
                // We're not setting 'action' here, because
                // ajaxUrlRequest/loadScript might error out and therefore
                // needs to stay on the prefs panel and not clear
                // the widget until after we try loading
                // the new URL/new script.
                
                // here zero means start of string (-1 would be false)
                if ( url.indexOf( 'script:' ) == 0 )
                {
                    self.loadScript( url, isNew );
                }
                else
                {
                    self.ajaxUrlRequest( url, isNew );
                }
                return;
        }

        if ( action )
        {
            if ( isNew )
            {
                self.newAction( action, sweep );
            }
            else
            {
                self.clearActions();
            
                ( action )();
            }   
        }
    }
    
    
    // Call this the first time you start an entirely
    // new chain of actions, such as from the flip
    // side of the widget using the UI
    this.newAction = function ( action, sweep )
    {
        self.clearActions();
        
        if ( sweep )
        {
            setTimeout( function () { self.clear( action ); }, 1000 );
        }
        else
        {
            setTimeout( action, 1000 );
        }

        self.hidePrefs();
    }
    
    
    this.clearActions = function ()
    {
        // Un-pause
        self.paused = false;
        
        // Stop that pesky clock!
        self.clockOn = 0;
        
        // Stop that pesky scroll!
        self.scrollOn = 0;
        
        // Let us get new clicks
        self.clickedLEDindex = -1;

        // Stop the loop so we can start a new one
        clearTimeout( self.loopTimer );

        // Reset this
        self.onclickURL = null;
    }
    
    
    this.parseScriptUrl = function ( url )
    {
        var command = '';
        var args = '';
        
        // Tailing space on '.led ' important!
        var endOfCommand = url.indexOf( '.led ' );
        
        // There is no in '.led ' in the string, so assume url is command with no args
        if ( endOfCommand < 0 )
        {
            command = url;
        }
        // Separate command and arguments
        else
        {
            command = url.substr( 0, endOfCommand + 4 );
            
            // Trimmed
            args = url.substr( endOfCommand + 4 ).replace( /^\s+|\s+$/g, '' );
        }
        
        // Crop that script: off the front
        command = command.replace( /^script:/, '' );
        
        // Convert silly %20 in command to space
        command = command.replace( /%20/g, ' ' );

        // All unescaped spaces in command are escaped
        command = command.replace( /([^\\]) /g, '$1\\ ' );
        
        // Is the command safe / valid?
        if ( url.search( /^script:/ ) < 0 || command.search( /^([-a-zA-Z0-9~_.\/]|\\\s)+\.led$/ ) < 0 )
        {
            return false;
        }
        
        if ( args )
        {
            // If a string has a ! but no single quotes, it has to be single-quoted
            // before it is passed to shell as command line arg
            if ( args.indexOf( '!' ) >= 0 && args.indexOf( "'" ) == -1 )
            {
                args = "'" + args + "'";
            }
            // This string either has no ! or has single quotes, so it has to be double-quoted
            // before it is passed to shell as command line arg
            else
            {
                // and we have to escape " and ` and $
                // and we should probably escape \ (but we have to do \ first!!!)
                args = args.replace( /\\/g, '\\\\' );
                args = args.replace( /"/g, '\\"' );
                args = args.replace( /`/g, '\\`' );
                args = args.replace( /\$/g, '\\$' );
                args = '"' + args + '"';
            }
        }
        
        // And handle onClick info!
        if ( self.clickedLEDindex >= 0 )
        {
            args = args + ' i=' + self.clickedLEDindex.toString();
        }
        
        return new Array( command, args );
    }
    
    
    this.loadScript = function ( url, isNew )
    {        
        if ( window.widget )
        {
            var parsedUrl = self.parseScriptUrl( url );

            if ( !parsedUrl || !parsedUrl[0] )
            {
                self.showError( 'Bad script URL.  Use "script:/path/to/script.led [optional text]"' );
            }
            else
            {
                var command = parsedUrl[0];
                var args = parsedUrl[1];
                
                if ( self.systemObj )
                {
                    self.systemObj.cancel();
                }

                // Explictly setting to executable will save many people a headache!
                widget.system( '/bin/chmod u+x ' + command, function () {

                    self.systemObj = widget.system( command + ' ' + args, function () { self.scriptLoaded( command, args, isNew ); } );

                    if ( !self.systemObj )
                    {
                        self.showError( 'Failed to load script: <span style="cursor: help; border-bottom: 1px dotted white;" title="' + command + '">' + command.substring( 0, 24 ) + '...</span>' );
                    }
                    else
                    {
                        self.systemObj.onreaderror = function ( error ) { self.showError( 'Script: <span style="cursor: help; border-bottom: 1px dotted white;" title="' + command + '">' + command.substring( 0, 24 ) + '...</span><br>Error: <span style="cursor: help; border-bottom: 1px dotted white;" title="' + error + '">' + error.substring( 0, 24 ) + '...</span>' ); };
                    }
                
                } );            
            }
        }
    }

    
    this.scriptLoaded = function ( command, args, isNew )
    {
        // We've got output, try to make sense of it
        var rawListItems = self.systemObj.outputString.split( /<\/li>/ );

        // We never have and never will have less than 8 items
        if ( !rawListItems || rawListItems.length < 8 )
        {
            self.showError( 'Could not understand data: <span style="cursor: help; border-bottom: 1px dotted white;" title="' + command + ' ' + args + '">' + ( command + ' ' + args ).substring( 0, 24 ) + '...</span>' );
        }
        // This case will allow us to suggest upgrading in the future should
        // the number of fields increase beyond what this version accepts.
        else if ( rawListItems.length > 8 )
        {
            self.showError( 'Too many fields in data: <span style="cursor: help; border-bottom: 1px dotted white;" title="' + command + ' ' + args + '">' + ( command + ' ' + args ).substring( 0, 24 ) + '...</span><br>Try upgrading your widget.' );
        }
        // Here we handle the current number of fields, and also
        // do backwards compatability.
        else if ( rawListItems.length == 8 )
        {
            if ( isNew )
            {
                self.newAction( function () { self.execNewData( rawListItems ); }, 1 );
            }
            else
            {
                self.execNewData( rawListItems );
            }
        }
    }
    
    
    this.ajaxUrlRequest = function ( url, isNew )
    {
        // (Hopefully) Prevent caching
        if ( url.indexOf( '?' ) >= 0 )
        {
            url = url + '&t=' + Math.round( Math.random() * 10000 ).toString();
        }
        else
        {
            url = url + '?t=' + Math.round( Math.random() * 10000 ).toString();
        }
        
        // And handle onClick info!
        if ( self.clickedLEDindex >= 0 )
        {
            url = url + '&i=' + self.clickedLEDindex.toString();
        }
        
        if ( self.XML_Request )
        {
            self.XML_Request.abort();
        }
        
        self.XML_Request = new XMLHttpRequest();
        self.XML_Request.onreadystatechange = function () { self.ajaxStateChange( url, isNew ) };
        self.XML_Request.open( 'GET', url, true );
        self.XML_Request.send( null );
    }
    
    
    this.ajaxStateChange = function ( url, isNew )
    {
        // self.XML_Request.status is undefined for local URL requests if no local HTTP server is running
        if ( self.XML_Request.readyState == 4 && ( !self.XML_Request.status || self.XML_Request.status == 200 ) )
        {
            if ( !self.XML_Request.responseText )
            {
                self.showError( '2. Failed to load URL: <span style="cursor: help; border-bottom: 1px dotted white;" title="' + url + '">' + url.substring( 0, 24 ) + '...</span>' );
            }
            else
            {
                // We've got a non-empty response, try to make sense of it
                var rawListItems = self.XML_Request.responseText.split( /<\/li>/ );

                // Handle people using file:// for .led files (if the data is bad)
                if ( rawListItems.length != 8 && url.indexOf( 'file://' ) == 0 && url.search( /\.led\?|\.led\s+|\.led$/ ) >= 0 )
                {
                    var suggest = url.replace( /^file:\/\//, 'script:' ).replace( /\?.*$/, '' ).replace( /%20/g, ' ' );
                    self.showError( 'Error! Try using script: instead. <span style="cursor: help; border-bottom: 1px dotted white;" title="' + suggest + '">' + suggest.substring( 0, 24 ) + '...</span>' );
                }
                // We never have and never will have less than 8 items
                else if ( rawListItems.length < 8 )
                {
                    self.showError( 'Could not understand data: <span style="cursor: help; border-bottom: 1px dotted white;" title="' + url + '">' + url.substring( 0, 24 ) + '...</span>' );
                }
                // This case will allow us to suggest upgrading in the future should
                // the number of fields increase beyond what this version accepts.
                else if ( rawListItems.length > 8 )
                {
                    self.showError( 'Too many fields in data: <span style="cursor: help; border-bottom: 1px dotted white;" title="' + url + '">' + url.substring( 0, 24 ) + '...</span><br>Try upgrading your widget.' );
                }
                // Here we handle the current number of fields, and also
                // do backwards compatability.
                else if ( rawListItems.length == 8 )
                {
                    if ( isNew )
                    {
                        self.newAction( function () { self.execNewData( rawListItems ); }, 1 );
                    }
                    else
                    {
                        self.execNewData( rawListItems );
                    }
                }
            }
        }
        else if ( self.XML_Request.readyState == 4 )
        {
            self.showError( '1. Failed to load URL: <span style="cursor: help; border-bottom: 1px dotted white;" title="' + url + '">' + url.substring( 0, 24 ) + '...</span>' );
        }
    }
    
    
    this.execNewData = function ( rawListItems )
    {
        var item;

        var errors = new Array();

        var hcount;
        var vcount;
        var bits;
        var nextURL;
        var refresh;
        var onclickURL;
        
        // Stop previous frame / clear previous frame data!
        self.clearActions();
        
        for ( var i = 0; i < rawListItems.length; i++ )
        {
            item = rawListItems[i].split( /<li>/ )[1];

            switch ( i )
            {
                // new hcount 
                case 0:
                    hcount = parseInt( item );
                    
                    if ( hcount < 1 || hcount > 64 )
                    {
                        errors.push( 'Horizontal # of LEDs out of bounds.' );
                    }
                    break;

                // new vcount
                case 1:
                    vcount = parseInt( item );
                    
                    if ( vcount < 1 || vcount > 64 )
                    {
                        errors.push( 'Vertical # of LEDs out of bounds.' );
                    }
                    break;

                // bit field
                case 2:
                    bits = item;
                    break;

                // next url
                case 3:
                    nextURL = item;
                    break;

                // how long until refresh
                case 4:
                    if ( item.indexOf( '[persistent]' ) >= 0 )
                    {
                        self.persistent = true;
                    }
                    else
                    {
                        self.persistent = false;
                    }
                    
                    refresh = parseInt( item );
                    
                    if ( refresh < 50 && refresh != 0 )
                    {
                        errors.push( 'Refresh out of bounds.' );
                    }
                    break;

                // onclick url
                case 5:
                    onclickURL = item;
                    break;

                // url to open in a browser
                case 6:
                    if ( window.widget && item )
                    {
                        widget.openURL( item );
                    }
                    break;
                
                // last bit is </ul> junk
                case 7:
                    break;
            }
        }
        
        if ( errors.length > 0 )
        {
            self.showError( errors.join( ' ' ) );
        }
        else
        {
            // If the size has changed, re-init
            if ( self.hcount != hcount || self.vcount != vcount )
            {
                self.init( hcount, vcount );
            }

            // Blit our bits
            self.blit( bits );

            // Setup onclick handling
            // Does rewriting a function leak memory?
            self.onclickURL = onclickURL;

            // Setup next round
            if ( nextURL && refresh != 0 && ( self.persistent || !self.paused ) )
            {
                self.nextAction = function () { self.processUrl( nextURL, 0 ); };
                self.loopTimer = setTimeout( self.nextAction, refresh );
            }
            // Sometimes the panel is click-driven and there is no timed loop
            // and we don't want a resume() call running some random action,
            // so we null the nextAction out here.
            else
            {
                self.nextAction = null;
            }
        }
    }
    
    
    this.pause = function ()
    {
        clearTimeout( self.loopTimer );
        self.paused = true;
    }
    
    
    this.resume = function ()
    {
        // Simple condition prevents multiple resumes /
        // multiple calls of nextAction function
        if ( self.paused )
        {
            self.paused = false;
        
            if ( self.nextAction )
            {
                ( self.nextAction )();
            }
        }
    }
    
    
    this.clickHandler = function ( i )
    {
        // We must save url temp var because the call to clearActions()
        // below will delete self.onclickURL.  We call clearActions() to stop
        // the Panel loop timer and make it handle our click URL exclusively.
        var url = self.onclickURL;
        
        // No click pending
        if ( self.clickedLEDindex < 0 && url )
        {
            self.clearActions();
            
            // set this AFTER we clear!
            self.clickedLEDindex = i;
        
            // Process onClick URL
            self.processUrl( url, 0 );
        }
    }


    // The 'action' argument is the action that should take place
    // after the clear has finished... before this was added, a
    // clear running asynchronously would sometimes wipe out half
    // a frame that had been drawn before the clear was finished.
    this.clear = function ( action )
    {
        self.clri = ( self.clri + self.hcount );

        var i;
        
        // Clear rows before our sweep row
        if ( self.clri >= self.hcount )
        {
            for ( i = self.clri - self.hcount; i < self.clri; i++ )
            {
                self.off( i );
            }
        }

        // Draw the sweep row
        for ( i = self.clri; i < self.clri + self.hcount && i < self.LEDs.length; i++ )
        {
            self.on( i );
        }

        // Keep clearing if we aren't done clearing all LEDs
        if ( self.clri < self.hcount * self.vcount )
        {
            setTimeout( function () { self.clear( action ); }, 50 );
        }   
        // Done clearing
        else
        {
            // Reset for next clear
            self.clri = -1 * self.hcount;
            
            // Is there a pending action?
            if ( action )
            {
                ( action )();
            }
        }
    }
    
    
    this.on = function ( i )
    {
        self.LEDs[i].style.backgroundPosition = "20px 0";
        self.states[i] = 1;
    }
    
    
    this.off = function ( i )
    {
        self.LEDs[i].style.backgroundPosition = "0 0";
        self.states[i] = 0;
    }
    
    
    this.toggle = function ( i )
    {
        if ( self.states[i] == 1 )
        {
            self.LEDs[i].style.backgroundPosition = "0 0";
            self.states[i] = 0;
        }
        else if ( self.states[i] == 0 )
        {
            self.LEDs[i].style.backgroundPosition = "20px 0";
            self.states[i] = 1;
        }
    }
    
    
    this.setState = function ( i, state )
    {
        if ( state == 1 )
        {
            self.LEDs[i].style.backgroundPosition = "20px 0";
            self.states[i] = 1;
        }
        else if ( state == 0 )
        {
            self.LEDs[i].style.backgroundPosition = "0 0";
            self.states[i] = 0;
        }
    }
    
    
    this.blit = function ( str )
    {
        k = 0;
        for ( var i = 0; i < str.length && k < self.LEDs.length; i++ )
        {
            if ( str[i] == '1' )
            {
                self.on( k );
                k++;
            }
            else if ( str[i] == '0' )
            {
                self.off( k );
                k++;
            }
        }
    }
    
    
    this.clockInit = function ()
    {
        if ( self.hcount != 7 || self.vcount != 7 )
        {
            self.init( 7, 7 );
        }
        
        self.clockOn = 1;
        self.clockTick();
    }
    
    
    this.clockTick = function ()
    {
        var date = new Date();

        var str = '';
        
        var dow = '';
        for ( var i = 0; i < 7; i++ )
        {
            if ( i == date.getDay() )
            {
                dow = dow + '1';
            }
            else
            {
                dow = dow + '0';
            }
        }
        
        str = self.padToWidth( self.makeBinStr( parseInt( date.getYear() ) - 100 ), 'right', 0 );
        str = str + self.padToWidth( self.makeBinStr( date.getMonth() + 1 ), 'right', 0 );
        str = str + dow;
        str = str + self.padToWidth( self.makeBinStr( date.getDate() ), 'right', 0 );
        str = str + self.padToWidth( self.makeBinStr( date.getHours() ), 'right', 0 );
        str = str + self.padToWidth( self.makeBinStr( date.getMinutes() ), 'right', 0 );
        str = str + self.padToWidth( self.makeBinStr( date.getSeconds() ), 'right', 0 );
        
        self.blit( str );
        
        if ( self.clockOn && !self.paused )
        {
            clearTimeout( self.loopTimer );
            self.nextAction = function () { self.clockTick(); };
            self.loopTimer = setTimeout( self.nextAction, 1000 );
        }
    }
    
    
    this.padToWidth = function ( str, align, state )
    {
        if ( str.length >= self.hcount )
        {
            return str;
        }
        
        var pad = '';
        for ( var i = 0; i < self.hcount - str.length; i++ )
        {
            pad = pad + state.toString();
        }
        
        if ( align == 'left' )
        {            
            return str + pad;
        }
        
        if ( align == 'right' )
        {            
            return pad + str;
        }
    }
    
    
    this.makeBinStr = function ( n )
    {
        if ( n <= 0 )
        {
            return '0';
        }
        
        var str = '';
        
        var b = 1;
        
        while ( b <= n )
        {
            if ( n & b )
            {
                str = '1' + str;
            }
            else
            {
                str = '0' + str;
            }
            
            b = b * 2;
        }
        
        return str;
    }
    
    
    this.intro = function ()
    {
        self.invader();
        
        self.scrollInit( 50, 2500 );
        setTimeout( function () { self.clear(); }, 2000 );
    }
    
    
    this.invader = function ()
    {
        if ( self.hcount != 16 || self.vcount != 16 )
        {
            self.init( 16, 16 );
        }
        
        self.blit( '0000000000000000000001000010000000000010010000000000011111100000000011011011000000011111111110000001011111101000000101000010100000000010010000000000000000000000010001110111000001000100010010000100011001001000010001000100100001110111011100100000000000000000' );
    }
    
    
    this.scrollInit = function ( delay, duration )
    {
        self.scrollOn = 1;
        self.scroll( delay, duration );
    }
    
    
    this.scroll = function ( delay, duration )
    {
        absDelay = Math.abs( delay );
        
        if ( delay < 0 )
        {
            var col = 0;
            var tempState;
            for ( var i = self.LEDs.length - 1; i >= 0; i-- )
            {
                if ( i % self.hcount == 0 )
                {
                    self.setState( i, tempState );
                }
                else
                {
                    if ( i % self.hcount == self.hcount - 1 )
                    {
                        tempState = self.states[i];
                    }

                    self.setState( i, self.states[i - 1] );
                }

            }
        }
        else
        {
            var col = 0;
            var tempState;
            for ( var i = 0; i < self.LEDs.length; i++ )
            {            
                if ( i % self.hcount == self.hcount - 1 )
                {
                    self.setState( i, tempState );
                }
                else
                {
                    if ( i % self.hcount == 0 )
                    {
                        tempState = self.states[i];
                    }

                    self.setState( i, self.states[i + 1] );
                }

            }
        }
        
        if ( self.scrollOn && duration == 0 )
        {
            clearTimeout( self.loopTimer );
            self.nextAction = function () { self.scroll( delay, 0 ); };
            self.loopTimer = setTimeout( self.nextAction, absDelay );
        }
        else if ( self.scrollOn && duration - absDelay > 0 )
        {
            clearTimeout( self.loopTimer );
            self.nextAction = function () { self.scroll( delay, duration - absDelay ); };
            self.loopTimer = setTimeout( self.nextAction, absDelay );
        }
    }
    
    
    this.showError = function ( error )
    {        
        if ( self.errorElement )
        {
            self.errorElement.innerHTML = error;
            self.errorElement.style.display = 'block';
            self.showPrefs();
        }
    }
    
    
    this.hideError = function ()
    {
        if ( self.errorElement )
        {
            self.errorElement.innerHTML = '';
            self.errorElement.style.display = 'none';
        }
    }
    
    
    this.showPrefs = function ()
    {
        if ( self.versionElement )
        {
            self.checkVersion();
        }
        
        if ( window.widget )
        {
            // This freezes the currently visible layer
            widget.prepareForTransition( 'ToBack' );
        }
        
        if ( self.frontElement && self.backElement )
        {
            self.frontElement.style.display = 'none';
            self.backElement.style.display = 'block';
        }
        
        if ( window.widget )
        {
        	setTimeout( 'widget.performTransition();', 0 );
        }
    }


    this.hidePrefs = function ()
    {
        if ( window.widget )
        {
            // This freezes the currently visible layer
            widget.prepareForTransition( 'ToFront' );
        }
        
        if ( self.frontElement && self.backElement )
        {
            self.frontElement.style.display = 'block';
            self.backElement.style.display = 'none';
        }
        
        if ( window.widget )
        {
        	setTimeout( 'widget.performTransition();', 0 );
        }
    }
}
